"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = require("./config");
const symbols_1 = require("./symbols");
const utils_1 = require("./utils");
const valit_1 = require("./valit");
const vality_1 = require("./vality");
vality_1.vality.array = (0, valit_1.valit)("array", (e) => (value, options, path) => {
    const fn = (0, utils_1.enyToGuardFn)(e);
    if (!Array.isArray(value)) {
        if (!config_1.config.strict) {
            const res = fn(value, [...path, 0], value);
            if (res.valid) {
                return { valid: true, data: [res.data], errors: [] };
            }
        }
        return {
            valid: false,
            data: undefined,
            errors: [{ message: "vality.array.base", path, options, value }],
        };
    }
    const data = [];
    const errors = [];
    for (let k = 0; k < value.length; k++) {
        const res = fn(value[k], [...path, k], value);
        if (!res.valid) {
            errors.push(...res.errors);
            if (options.bail)
                break;
        }
        else {
            data.push(res.data);
        }
    }
    if (errors.length === 0)
        return { valid: true, data, errors: [] };
    return { valid: false, data: undefined, errors };
}, {
    minLength: (val, o) => val.length >= o,
    maxLength: (val, o) => val.length <= o,
}, {
    bail: false,
});
vality_1.vality.object = (0, valit_1.valit)("object", (e) => (value, options, path) => {
    if (typeof value !== "object" || value === null)
        return {
            valid: false,
            data: undefined,
            errors: [{ message: "vality.object.base", path, options, value }],
        };
    const data = {};
    const errors = [];
    for (const k in e) {
        const ek = e[k];
        if (typeof ek === "function" &&
            (ek === null || ek === void 0 ? void 0 : ek[symbols_1._type]) === "readonly")
            continue;
        const res = (0, utils_1.enyToGuardFn)(ek)(value[k], [...path, k], value);
        if (!res.valid) {
            errors.push(...res.errors);
            if (options.bail)
                break;
        }
        else {
            data[k] = res.data;
        }
    }
    for (const k in value) {
        const ek = e[k];
        if (ek === undefined ||
            (typeof ek === "function" &&
                (ek === null || ek === void 0 ? void 0 : ek[symbols_1._type]) === "readonly")) {
            errors.push({
                message: "vality.object.extraProperty",
                path: [...path, k],
                options,
                value: value[k],
            });
            if (options.bail)
                break;
        }
    }
    if (errors.length === 0)
        return { valid: true, data: data, errors: [] };
    return { valid: false, data: undefined, errors };
}, {}, {
    bail: false,
});
vality_1.vality.optional = (0, valit_1.valit)("optional", (e) => (val, _options, path, parent) => {
    const enyVal = (0, utils_1.enyToGuardFn)(e)(val, path, parent);
    if (enyVal.valid)
        return enyVal;
    if (val === undefined)
        return { valid: true, data: undefined, errors: [] };
    if (!config_1.config.strict && val === null)
        return { valid: true, data: undefined, errors: [] };
    return enyVal;
});
vality_1.vality.enum = (0, valit_1.valit)("enum", (...es) => (value, options, path, parent) => {
    for (const e of es) {
        const res = (0, utils_1.enyToGuardFn)(e)(value, path, parent);
        if (res.valid)
            return res;
    }
    return {
        valid: false,
        data: undefined,
        errors: [{ message: "vality.enum.base", path, options, value }],
    };
});
vality_1.vality.tuple = (0, valit_1.valit)("tuple", (...es) => (value, options, path) => {
    if (!Array.isArray(value))
        return {
            valid: false,
            data: undefined,
            errors: [{ message: "vality.tuple.base", path, options, value }],
        };
    const data = [];
    const errors = [];
    for (let i = 0; i < es.length; i++) {
        const res = (0, utils_1.enyToGuardFn)(es[i])(value[i], [...path, i], value);
        if (!res.valid) {
            errors.push(...res.errors);
            if (options.bail)
                break;
        }
        else {
            data[i] = res.data;
        }
    }
    for (let i = es.length; i < value.length; i++) {
        errors.push({
            message: "vality.tuple.extraProperty",
            path: [...path, i],
            options,
            value: value[i],
        });
        if (options.bail)
            break;
    }
    if (errors.length === 0)
        return { valid: true, data, errors: [] };
    return { valid: false, data: undefined, errors };
}, {}, {
    bail: false,
});
vality_1.vality.readonly = (0, valit_1.valit)("readonly", (e) => (val, _options, path) => {
    if (val === undefined)
        return { valid: true, data: undefined, errors: [] };
    return {
        valid: false,
        data: undefined,
        errors: [
            {
                message: "vality.readonly.base",
                path,
                options: {},
                value: val,
            },
        ],
    };
});
vality_1.vality.and = (0, valit_1.valit)("and", (...es) => (value, options, path, parent) => {
    if (typeof value !== "object" || value === null)
        return {
            valid: false,
            data: undefined,
            errors: [{ message: "vality.and.base", path, options, value }],
        };
    const data = {};
    const errors = [];
    const handleEs = (ess) => {
        for (let i = 0; i < ess.length; i++) {
            const eGuard = (0, utils_1.enyToGuard)(ess[i]);
            const typeOfGuard = eGuard[symbols_1._name];
            let res = undefined;
            switch (typeOfGuard) {
                case "object": {
                    const objectValue = {};
                    for (const k in eGuard[symbols_1._validate][symbols_1._type][0]) {
                        Object.assign(objectValue, {
                            [k]: value[k],
                        });
                    }
                    res = eGuard[symbols_1._validate](objectValue, path, parent);
                    break;
                }
                case "enum":
                    for (const e of eGuard[symbols_1._validate][symbols_1._type]) {
                        const enumMemberGuardFn = (0, utils_1.enyToGuardFn)(e);
                        const enumMemberValue = {};
                        for (const k in enumMemberGuardFn[symbols_1._type][0]) {
                            Object.assign(enumMemberValue, {
                                [k]: value[k],
                            });
                        }
                        res = enumMemberGuardFn(enumMemberValue, path, parent);
                        if (res.valid)
                            break;
                    }
                    if (!res)
                        res = {
                            valid: false,
                            data: undefined,
                            errors: [
                                { message: "vality.enum.base", path, options, value },
                            ],
                        };
                    break;
                case "and":
                    handleEs(eGuard[symbols_1._validate][symbols_1._type]);
                    break;
                default:
                    throw new Error("vality.and: Unexpected type of guard: " + typeOfGuard);
            }
            if (!res)
                continue;
            if (!res.valid) {
                errors.push(...res.errors);
                if (options.bail)
                    break;
            }
            else {
                Object.assign(data, res.data);
            }
        }
    };
    handleEs(es);
    if (errors.length !== 0)
        return { valid: false, data: undefined, errors };
    const gotKeys = Object.keys(data).length;
    const expectedKeys = Object.keys(value).length;
    if (gotKeys < expectedKeys) {
        return {
            valid: false,
            data: undefined,
            errors: [
                { message: "vality.and.extraProperties", path, options, value },
            ],
        };
    }
    else if (gotKeys > expectedKeys) {
        throw new Error("This can't happen");
    }
    return { valid: true, data, errors: [] };
}, {}, {
    bail: false,
});
vality_1.vality.dict = (0, valit_1.valit)("dict", (k, v) => (value, options, path, parent) => {
    if (typeof value !== "object" || value === null)
        return {
            valid: false,
            data: undefined,
            errors: [{ message: "vality.dict.base", path, options, value }],
        };
    const keyGuard = (0, utils_1.enyToGuard)(k);
    const typeOfKey = keyGuard[symbols_1._name];
    if (typeOfKey === "literal" ||
        typeOfKey === "string" ||
        typeOfKey === "number") {
        return vality_1.vality
            .dict(vality_1.vality.enum(keyGuard), v)(options)[symbols_1._validate](value, path, parent);
    }
    const keysGuards = keyGuard[symbols_1._validate][symbols_1._type].map(utils_1.enyToGuard);
    const literalKeys = keysGuards.filter((g) => g[symbols_1._name] === "literal");
    const errors = [];
    for (const key in value) {
        if (!keyGuard[symbols_1._validate](key, [...path, key], value).valid) {
            errors.push({
                message: "vality.dict.invalidProperty",
                path: [...path, key],
                options,
                value: key,
            });
            if (options.bail)
                break;
        }
    }
    if (errors.length)
        return { valid: false, data: undefined, errors };
    for (const literalKeyGuard of literalKeys) {
        if (!Object.keys(value).some((k) => literalKeyGuard[symbols_1._validate](k, [...path, k], value).valid)) {
            if (!literalKeyGuard[symbols_1._validate](k, path, value).valid) {
                errors.push({
                    message: "vality.dict.missingProperty",
                    path: [
                        ...path,
                        literalKeyGuard[symbols_1._validate][symbols_1._type][0][symbols_1._type],
                    ],
                    options,
                    value: undefined,
                });
                if (options.bail)
                    break;
            }
        }
    }
    if (errors.length)
        return { valid: false, data: undefined, errors };
    const valueGuardFn = (0, utils_1.enyToGuardFn)(v);
    const data = {};
    for (const key in value) {
        const res = valueGuardFn(value[key], [...path, key], value);
        if (!res.valid) {
            errors.push(...res.errors);
            if (options.bail)
                break;
        }
        else {
            data[key] = res.data;
        }
    }
    if (errors.length)
        return { valid: false, data: undefined, errors };
    return { valid: true, data, errors: [] };
}, {}, {
    bail: false,
});
